@@ -1,8 +1,7 @@
 /*
  * The main header file for SIP.
  *
- * Copyright (c) 2008
- * 	Phil Thompson <phil@river-bank.demon.co.uk>
+ * Copyright (c) 2009 Riverbank Computing Limited <info@riverbankcomputing.com>
  * 
  * This file is part of SIP.
  * 
@@ -40,7 +39,7 @@
 
 /* For convenience. */
 
-#define classBaseName(cd)   ((cd)->iff->name->text)
+#define classBaseName(cd)   scopedNameTail((cd)->iff->fqcname)
 #define classFQCName(cd)    ((cd)->iff->fqcname)
 
 
@@ -49,6 +48,7 @@
 #define MOD_HAS_DELAYED_DTORS   0x0001  /* It has a class with a delayed dtor. */
 #define MOD_IS_CONSOLIDATED     0x0002  /* It is a consolidated module. */
 #define MOD_IS_COMPOSITE        0x0004  /* It is a composite module. */
+#define MOD_IS_TRANSFORMED      0x0008  /* It's types have been transformed. */
 
 #define hasDelayedDtors(m)  ((m)->modflags & MOD_HAS_DELAYED_DTORS)
 #define setHasDelayedDtors(m)   ((m)->modflags |= MOD_HAS_DELAYED_DTORS)
@@ -57,6 +57,8 @@
 #define isComposite(m)      ((m)->modflags & MOD_IS_COMPOSITE)
 #define setIsComposite(m)   ((m)->modflags |= MOD_IS_COMPOSITE)
 #define isContainer(m)      ((m)->modflags & (MOD_IS_CONSOLIDATED | MOD_IS_COMPOSITE))
+#define setIsTransformed(m) ((m)->modflags |= MOD_IS_TRANSFORMED)
+#define isTransformed(m)    ((m)->modflags & MOD_IS_TRANSFORMED)
 
 
 /* Handle section flags. */
@@ -79,15 +81,18 @@
 #define CLASS_DTOR_RELEASE_GIL  0x00004000  /* The dtor releases the GIL. */
 #define CLASS_IS_PROTECTED  0x00008000  /* It is protected. */
 #define CLASS_IS_PROTECTED_SAV  0x00010000  /* It is protected (saved). */
-#define CLASS_IS_RENAMED    0x00020000  /* It has a different Python name. */
-#define CLASS_IS_INCOMPLETE 0x00040000  /* The specification is incomplete. */
-#define CLASS_CAN_CREATE    0x00080000  /* It has usable ctors. */
-#define CLASS_IS_EXTERNAL   0x00100000  /* It is external. */
-#define CLASS_IS_DELAYED_DTOR   0x00200000  /* The dtor is delayed. */
-#define CLASS_NO_DEFAULT_CTORS  0x00400000  /* Don't create default ctors. */
-#define CLASS_QOBJECT_SUB   0x00800000  /* It is derived from QObject. */
-#define CLASS_DTOR_HOLD_GIL 0x01000000  /* The dtor holds the GIL. */
-#define CLASS_QT_META_TYPE  0x02000000  /* Register as a Qt meta type. */
+#define CLASS_IS_INCOMPLETE 0x00020000  /* The specification is incomplete. */
+#define CLASS_CAN_CREATE    0x00040000  /* It has usable ctors. */
+#define CLASS_IS_EXTERNAL   0x00080000  /* It is external. */
+#define CLASS_IS_DELAYED_DTOR   0x00100000  /* The dtor is delayed. */
+#define CLASS_NO_DEFAULT_CTORS  0x00200000  /* Don't create default ctors. */
+#define CLASS_QOBJECT_SUB   0x00400000  /* It is derived from QObject. */
+#define CLASS_DTOR_HOLD_GIL 0x00800000  /* The dtor holds the GIL. */
+#define CLASS_ASSIGN_HELPER 0x01000000  /* Generate an assignment helper. */
+#define CLASS_NO_QMETAOBJECT    0x02000000  /* It has no QMetaObject. */
+#define CLASS_IS_TEMPLATE   0x04000000  /* It is a template class. */
+#define CLASS_IS_DEPRECATED 0x08000000  /* It is deprecated. */
+#define CLASS_CANNOT_COPY   0x10000000  /* It cannot be copied. */
 
 #define hasSigSlots(cd)     ((cd)->classflags & CLASS_HAS_SIGSLOTS)
 #define setHasSigSlots(cd)  ((cd)->classflags |= CLASS_HAS_SIGSLOTS)
@@ -108,8 +113,6 @@
 #define resetWasProtectedClass(cd)  ((cd)->classflags &= ~CLASS_IS_PROTECTED_SAV)
 #define isReleaseGILDtor(c) ((cd)->classflags & CLASS_DTOR_RELEASE_GIL)
 #define setIsReleaseGILDtor(c)  ((cd)->classflags |= CLASS_DTOR_RELEASE_GIL)
-#define isRenamedClass(cd)  ((cd)->classflags & CLASS_IS_RENAMED)
-#define setIsRenamedClass(cd)   ((cd)->classflags |= CLASS_IS_RENAMED)
 #define isIncomplete(cd)    ((cd)->classflags & CLASS_IS_INCOMPLETE)
 #define setIsIncomplete(cd) ((cd)->classflags |= CLASS_IS_INCOMPLETE)
 #define canCreate(cd)       ((cd)->classflags & CLASS_CAN_CREATE)
@@ -125,8 +128,17 @@
 #define setIsQObjectSubClass(cd)    ((cd)->classflags |= CLASS_QOBJECT_SUB)
 #define isHoldGILDtor(c)    ((cd)->classflags & CLASS_DTOR_HOLD_GIL)
 #define setIsHoldGILDtor(c) ((cd)->classflags |= CLASS_DTOR_HOLD_GIL)
-#define registerQtMetaType(c)   ((cd)->classflags & CLASS_QT_META_TYPE)
-#define setRegisterQtMetaType(c)    ((cd)->classflags |= CLASS_QT_META_TYPE)
+#define assignmentHelper(c) ((cd)->classflags & CLASS_ASSIGN_HELPER)
+#define setAssignmentHelper(c)      ((cd)->classflags |= CLASS_ASSIGN_HELPER)
+#define noPyQt4QMetaObject(c)       ((cd)->classflags & CLASS_NO_QMETAOBJECT)
+#define setPyQt4NoQMetaObject(c)    ((cd)->classflags |= CLASS_NO_QMETAOBJECT)
+#define isTemplateClass(cd) ((cd)->classflags & CLASS_IS_TEMPLATE)
+#define setIsTemplateClass(cd)  ((cd)->classflags |= CLASS_IS_TEMPLATE)
+#define resetIsTemplateClass(cd)    ((cd)->classflags &= ~CLASS_IS_TEMPLATE)
+#define isDeprecatedClass(cd)   ((cd)->classflags & CLASS_IS_DEPRECATED)
+#define setIsDeprecatedClass(cd)    ((cd)->classflags |= CLASS_IS_DEPRECATED)
+#define cannotCopy(cd)      ((cd)->classflags & CLASS_CANNOT_COPY)
+#define setCannotCopy(cd)   ((cd)->classflags |= CLASS_CANNOT_COPY)
 
 #define isPublicDtor(cd)    ((cd)->classflags & SECT_IS_PUBLIC)
 #define setIsPublicDtor(cd) ((cd)->classflags |= SECT_IS_PUBLIC)
@@ -143,6 +155,7 @@
 #define CTOR_CAST           0x00000400  /* The ctor is a cast. */
 #define CTOR_HOLD_GIL       0x00000800  /* The ctor holds the GIL. */
 #define CTOR_XFERRED        0x00001000  /* Ownership is transferred. */
+#define CTOR_IS_DEPRECATED  0x00002000  /* The ctor is deprecated. */
 
 #define isPublicCtor(c)     ((c)->ctorflags & SECT_IS_PUBLIC)
 #define setIsPublicCtor(c)  ((c)->ctorflags |= SECT_IS_PUBLIC)
@@ -160,20 +173,24 @@
 #define setIsHoldGILCtor(c) ((c)->ctorflags |= CTOR_HOLD_GIL)
 #define isResultTransferredCtor(c)  ((c)->ctorflags & CTOR_XFERRED)
 #define setIsResultTransferredCtor(c)   ((c)->ctorflags |= CTOR_XFERRED)
+#define isDeprecatedCtor(c) ((c)->ctorflags & CTOR_IS_DEPRECATED)
+#define setIsDeprecatedCtor(c)  ((c)->ctorflags |= CTOR_IS_DEPRECATED)
 
 
 /* Handle member flags. */
 
 #define MEMBR_NUMERIC       0x0001      /* It is a numeric slot. */
+#define MEMBR_NO_ARG_PARSER 0x0002      /* Don't generate an argument parser. */
 
 #define isNumeric(m)        ((m)->memberflags & MEMBR_NUMERIC)
 #define setIsNumeric(m)     ((m)->memberflags |= MEMBR_NUMERIC)
+#define noArgParser(m)      ((m)->memberflags & MEMBR_NO_ARG_PARSER)
+#define setNoArgParser(m)   ((m)->memberflags |= MEMBR_NO_ARG_PARSER)
 
 
 /* Handle enum flags.  These are combined with the section flags. */
 
 #define ENUM_WAS_PROT       0x00000100  /* It was defined as protected. */
-#define ENUM_IS_RENAMED     0x00000200  /* It has been renamed. */
 
 #define isProtectedEnum(e)  ((e)->enumflags & SECT_IS_PROT)
 #define setIsProtectedEnum(e)   ((e)->enumflags |= SECT_IS_PROT)
@@ -182,8 +199,6 @@
 #define wasProtectedEnum(e) ((e)->enumflags & ENUM_WAS_PROT)
 #define setWasProtectedEnum(e)  ((e)->enumflags |= ENUM_WAS_PROT)
 #define resetWasProtectedEnum(e)    ((e)->enumflags &= ~ENUM_WAS_PROT)
-#define isRenamedEnum(e)    ((e)->enumflags & ENUM_IS_RENAMED)
-#define setIsRenamedEnum(e) ((e)->enumflags |= ENUM_IS_RENAMED)
 
 
 /* Handle hierarchy flags. */
@@ -215,6 +230,7 @@
 #define OVER_THIS_XFERRED   0x00200000  /* Ownership of this is transferred. */
 #define OVER_IS_GLOBAL      0x00400000  /* It is a global operator. */
 #define OVER_IS_COMPLEMENTARY   0x00800000  /* It is a complementary operator. */
+#define OVER_IS_DEPRECATED  0x01000000  /* It is deprecated. */
 
 #define isPublic(o)         ((o)->overflags & SECT_IS_PUBLIC)
 #define setIsPublic(o)      ((o)->overflags |= SECT_IS_PUBLIC)
@@ -261,6 +277,8 @@
 #define setIsGlobal(o)      ((o)->overflags |= OVER_IS_GLOBAL)
 #define isComplementary(o)  ((o)->overflags & OVER_IS_COMPLEMENTARY)
 #define setIsComplementary(o)   ((o)->overflags |= OVER_IS_COMPLEMENTARY)
+#define isDeprecated(o)     ((o)->overflags & OVER_IS_DEPRECATED)
+#define setIsDeprecated(o)  ((o)->overflags |= OVER_IS_DEPRECATED)
 
 
 /* Handle variable flags. */
@@ -289,9 +307,11 @@
 #define ARG_OUT             0x0400  /* It returns a result. */
 #define ARG_CONSTRAINED     0x0800  /* Suppress type conversion. */
 #define ARG_SINGLE_SHOT     0x1000  /* The slot is only ever fired once. */
+#define ARG_RESULT_SIZE     0x2000  /* It defines the result size. */
+#define ARG_KEEP_REF        0x4000  /* Keep a reference. */
 
 #define isReference(a)      ((a)->argflags & ARG_IS_REF)
-#define setIsReference(a)   ((a)-> argflags |= ARG_IS_REF)
+#define setIsReference(a)   ((a)->argflags |= ARG_IS_REF)
 #define resetIsReference(a) ((a)->argflags &= ~ARG_IS_REF)
 #define isConstArg(a)       ((a)->argflags & ARG_IS_CONST)
 #define setIsConstArg(a)    ((a)->argflags |= ARG_IS_CONST)
@@ -318,19 +338,22 @@
 #define setIsConstrained(a) ((a)->argflags |= ARG_CONSTRAINED)
 #define resetIsConstrained(a)   ((a)->argflags &= ~ARG_CONSTRAINED)
 #define isSingleShot(a)     ((a)->argflags & ARG_SINGLE_SHOT)
+#define isResultSize(a)     ((a)->argflags & ARG_RESULT_SIZE)
+#define setResultSize(a)    ((a)->argflags |= ARG_RESULT_SIZE)
+#define keepReference(a)    ((a)->argflags & ARG_KEEP_REF)
+#define setKeepReference(a) ((a)->argflags |= ARG_KEEP_REF)
 
 
 /* Handle name flags. */
 
 #define NAME_IS_USED        0x01    /* It is used in the main module. */
-#define NAME_IS_CLASS       0x02    /* It is the name of a class. */
+#define NAME_IS_SUBSTR      0x02    /* It is a substring of another. */
 
 #define isUsedName(n)       ((n)->nameflags & NAME_IS_USED)
 #define setIsUsedName(n)    ((n)->nameflags |= NAME_IS_USED)
 #define resetIsUsedName(n)  ((n)->nameflags &= ~NAME_IS_USED)
-#define isClassName(n)      ((n)->nameflags & NAME_IS_CLASS)
-#define setIsClassName(n)   ((n)->nameflags |= NAME_IS_CLASS)
-#define resetIsClassName(n) ((n)->nameflags &= ~NAME_IS_CLASS)
+#define isSubstring(n)      ((n)->nameflags & NAME_IS_SUBSTR)
+#define setIsSubstring(n)   ((n)->nameflags |= NAME_IS_SUBSTR)
 
 
 /* Handle virtual handler flags. */
@@ -345,6 +368,22 @@
 #define setIsTransferVH(vh) ((vh)->vhflags |= VH_TRANSFERS)
 
 
+/* Handle mapped type flags. */
+
+#define MT_NO_RELEASE       0x01    /* Do not generate a release function. */
+
+#define noRelease(mt)       ((mt)->mtflags & MT_NO_RELEASE)
+#define setNoRelease(mt)    ((mt)->mtflags |= MT_NO_RELEASE)
+
+
+/* Handle typedef flags. */
+
+#define TD_NO_TYPE_NAME     0x01    /* Do not use the typedef name. */
+
+#define noTypeName(td)      ((td)->tdflags & TD_NO_TYPE_NAME)
+#define setNoTypeName(td)   ((td)->tdflags |= TD_NO_TYPE_NAME)
+
+
 /* Slot types. */
 
 typedef enum {
@@ -362,6 +401,8 @@ typedef enum {
     repeat_slot,
     div_slot,
     mod_slot,
+    floordiv_slot,
+    truediv_slot,
     and_slot,
     or_slot,
     xor_slot,
@@ -374,6 +415,8 @@ typedef enum {
     irepeat_slot,
     idiv_slot,
     imod_slot,
+    ifloordiv_slot,
+    itruediv_slot,
     iand_slot,
     ior_slot,
     ixor_slot,
@@ -391,12 +434,13 @@ typedef enum {
     gt_slot,
     ge_slot,
     cmp_slot,
-    nonzero_slot,
+    bool_slot,
     neg_slot,
     pos_slot,
     abs_slot,
     repr_slot,
     hash_slot,
+    index_slot,
     no_slot
 } slotType;
 
@@ -450,7 +494,11 @@ typedef enum {
     cbool_type,
     sstring_type,
     wstring_type,
-    fake_void_type
+    fake_void_type,
+    ssize_type,
+    ascii_string_type,
+    latin1_string_type,
+    utf8_string_type
 } argType;
 
 
@@ -519,26 +567,13 @@ typedef struct _scopedNameDef {
 
 typedef struct _nameDef {
     int nameflags;                      /* The name flags. */
-    struct _moduleDef *module;          /* The main module. */
     const char *text;                   /* The text of the name. */
+    size_t len;                         /* The length of the name. */
+    size_t offset;                      /* The offset in the string pool. */
     struct _nameDef *next;              /* Next in the list. */
 } nameDef;
 
 
-/*
- * A node in the tree of classes used to determine the order in which the
- * classes need to be created.
- */
-
-typedef struct _nodeDef {
-    int ordered;                        /* Set if in order. */
-    struct _classDef *cd;               /* The class. */
-    struct _nodeDef *parent;            /* The parent. */
-    struct _nodeDef *child;             /* The first child. */
-    struct _nodeDef *next;              /* The next sibling. */
-} nodeDef;
-
-
 /* A literal code block. */
 
 typedef struct _codeBlock {
@@ -549,53 +584,6 @@ typedef struct _codeBlock {
 } codeBlock;
 
 
-/* A module definition. */
-
-typedef struct _moduleDef {
-    const char *fullname;               /* The full module name. */
-    const char *name;                   /* The module base name. */
-    int version;                        /* The module version. */
-    int modflags;                       /* The module flags. */
-    int qobjclass;                      /* QObject class, -1 if none. */
-    struct _memberDef *othfuncs;        /* List of other functions. */
-    struct _overDef *overs;             /* Global overloads. */
-    codeBlock *hdrcode;                 /* Header code. */
-    codeBlock *cppcode;                 /* Global C++ code. */
-    codeBlock *copying;                 /* Software license. */
-    codeBlock *preinitcode;             /* Pre-initialisation code. */
-    codeBlock *postinitcode;            /* Post-initialisation code. */
-    codeBlock *unitcode;                /* Compilation unit code. */
-    int modulenr;                       /* The module number. */
-    int parts;                          /* The number of parts generated. */
-    char *file;                         /* The filename. */
-    qualDef *qualifiers;                /* The list of qualifiers. */
-    nodeDef root;                       /* Root of class tree. */
-    int nrtimelines;                    /* The nr. of timelines. */
-    int nrclasses;                      /* The nr. of classes. */
-    int nrexceptions;                   /* The nr. of exceptions. */
-    int nrmappedtypes;                  /* The nr. of mapped types. */
-    int nrenums;                        /* The nr. of named enums. */
-    int nrtypedefs;                     /* The nr. of typedefs. */
-    int nrvirthandlers;                 /* The nr. of virtual handlers. */
-    struct _virtHandlerDef *virthandlers;   /* The virtual handlers. */
-    licenseDef *license;                /* The software license. */
-    struct _classDef *proxies;          /* The list of proxy classes. */
-    struct _moduleDef *container;       /* The container module, if any. */
-    struct _ifaceFileList *used;        /* Interface files used. */
-    struct _moduleListDef *allimports;  /* The list of all imports. */
-    struct _moduleListDef *imports;     /* The list of direct imports. */
-    struct _moduleDef *next;            /* Next in the list. */
-} moduleDef;
-
-
-/* An entry in a linked module list. */
-
-typedef struct _moduleListDef {
-    moduleDef *module;                  /* The module itself. */
-    struct _moduleListDef *next;        /* The next in the list. */
-} moduleListDef;
-
-
 /* The arguments to a throw specifier. */
 
 typedef struct _throwArgs {
@@ -644,6 +632,8 @@ typedef struct {
     int argflags;                       /* The argument flags. */
     int nrderefs;                       /* Nr. of dereferences. */
     valueDef *defval;                   /* The default value. */
+    int key;                            /* The optional /KeepReference/ key. */
+    struct _typedefDef *original_type;  /* The original type if typedef'd. */
     union {
         struct _signatureDef *sa;       /* If it is a function. */
         struct _templateDef *td;        /* If it is a template. */
@@ -673,6 +663,53 @@ typedef struct _fcallDef {
 } fcallDef;
 
 
+/* A module definition. */
+typedef struct _moduleDef {
+    nameDef *fullname;                  /* The full module name. */
+    const char *name;                   /* The module base name. */
+    int version;                        /* The module version. */
+    int modflags;                       /* The module flags. */
+    int qobjclass;                      /* QObject class, -1 if none. */
+    struct _memberDef *othfuncs;        /* List of other functions. */
+    struct _overDef *overs;             /* Global overloads. */
+    argType encoding;                   /* The default string encoding. */
+    nameDef *defmetatype;               /* The optional default meta-type. */
+    nameDef *defsupertype;              /* The optional default super-type. */
+    codeBlock *hdrcode;                 /* Header code. */
+    codeBlock *cppcode;                 /* Global C++ code. */
+    codeBlock *copying;                 /* Software license. */
+    codeBlock *preinitcode;             /* Pre-initialisation code. */
+    codeBlock *initcode;                /* Initialisation code. */
+    codeBlock *postinitcode;            /* Post-initialisation code. */
+    codeBlock *unitcode;                /* Compilation unit code. */
+    int parts;                          /* The number of parts generated. */
+    char *file;                         /* The filename. */
+    qualDef *qualifiers;                /* The list of qualifiers. */
+    argDef *types;                      /* The array of numbered types. */
+    int nrtypes;                        /* The number of numbered types. */
+    int nrtimelines;                    /* The nr. of timelines. */
+    int nrexceptions;                   /* The nr. of exceptions. */
+    int nrtypedefs;                     /* The nr. of typedefs. */
+    int nrvirthandlers;                 /* The nr. of virtual handlers. */
+    int next_key;                       /* The next key to allocate. */
+    struct _virtHandlerDef *virthandlers;   /* The virtual handlers. */
+    licenseDef *license;                /* The software license. */
+    struct _classDef *proxies;          /* The list of proxy classes. */
+    struct _moduleDef *container;       /* The container module, if any. */
+    struct _ifaceFileList *used;        /* Interface files used. */
+    struct _moduleListDef *allimports;  /* The list of all imports. */
+    struct _moduleListDef *imports;     /* The list of direct imports. */
+    struct _moduleDef *next;            /* Next in the list. */
+} moduleDef;
+
+
+/* An entry in a linked module list. */
+typedef struct _moduleListDef {
+    moduleDef *module;                  /* The module itself. */
+    struct _moduleListDef *next;        /* The next in the list. */
+} moduleListDef;
+
+
 /* An interface file definition. */
 
 typedef struct _ifaceFileDef {
@@ -697,7 +734,9 @@ typedef struct _ifaceFileList {
 /* A mapped type. */
 
 typedef struct _mappedTypeDef {
+    int mtflags;                        /* The mapped type flags. */
     argDef type;                        /* The type being mapped. */
+    nameDef *cname;                     /* The C/C++ name. */
     int mappednr;                       /* The mapped type number. */
     ifaceFileDef *iff;                  /* The interface file. */
     codeBlock *convfromcode;            /* Convert from C++ code. */
@@ -747,6 +786,7 @@ typedef struct _virtHandlerDef {
 /* A typedef definition. */
 
 typedef struct _typedefDef {
+    int tdflags;                        /* The typedef flags. */
     scopedNameDef *fqname;              /* The fully qualified name. */
     struct _classDef *ecd;              /* The enclosing class. */
     moduleDef *module;                  /* The owning module. */
@@ -816,9 +856,11 @@ typedef struct _enumMemberDef {
 
 typedef struct _enumDef {
     int enumflags;                      /* The enum flags. */
-    scopedNameDef *fqcname;             /* The name (may be NULL). */
     nameDef *pyname;                    /* The Python name (may be NULL). */
+    scopedNameDef *fqcname;             /* The C/C++ name (may be NULL). */
+    nameDef *cname;                     /* The C/C++ name (may be NULL). */
     int enumnr;                         /* The enum number. */
+    int enum_idx;                       /* The enum index within the module. */
     struct _classDef *ecd;              /* The enclosing class. */
     moduleDef *module;                  /* The owning module. */
     enumMemberDef *members;             /* The list of members. */
@@ -833,6 +875,7 @@ typedef struct _enumDef {
 typedef struct _memberDef {
     nameDef *pyname;                    /* The Python name. */
     int memberflags;                    /* The member flags. */
+    int membernr;                       /* The index in the method table. */
     slotType slot;                      /* The slot type. */
     moduleDef *module;                  /* The owning module. */
     struct _memberDef *next;            /* Next in the list. */
@@ -878,15 +921,16 @@ typedef struct _mroDef {
 
 typedef struct _classDef {
     int classflags;                     /* The class flags. */
-    int userflags;                      /* The user type flags. */
+    int pyqt4_flags;                    /* The PyQt4 specific flags. */
     int classnr;                        /* The class number. */
-    const char *pyname;                 /* The Python name. */
+    nameDef *pyname;                    /* The Python name. */
     ifaceFileDef *iff;                  /* The interface file. */
     struct _classDef *ecd;              /* The enclosing scope. */
     struct _classDef *real;             /* The real class if this is a proxy or extender. */
-    nodeDef *node;                      /* Position in class tree. */
     classList *supers;                  /* The parent classes. */
     mroDef *mro;                        /* The super-class hierarchy. */
+    nameDef *metatype;                  /* The meta-type. */
+    nameDef *supertype;                 /* The super-type. */
     templateDef *td;                    /* The instantiated template. */
     ctorDef *ctors;                     /* The constructors. */
     ctorDef *defctor;                   /* The default ctor. */
@@ -904,10 +948,12 @@ typedef struct _classDef {
     codeBlock *convtocode;              /* Convert to C++ code. */
     codeBlock *travcode;                /* Traverse code. */
     codeBlock *clearcode;               /* Clear code. */
-    codeBlock *readbufcode;             /* Read buffer code. */
-    codeBlock *writebufcode;            /* Write buffer code. */
-    codeBlock *segcountcode;            /* Segment count code. */
-    codeBlock *charbufcode;             /* Character buffer code. */
+    codeBlock *getbufcode;              /* Get buffer code (Python v3). */
+    codeBlock *releasebufcode;          /* Release buffer code (Python v3). */
+    codeBlock *readbufcode;             /* Read buffer code (Python v2). */
+    codeBlock *writebufcode;            /* Write buffer code (Python v2). */
+    codeBlock *segcountcode;            /* Segment count code (Python v2). */
+    codeBlock *charbufcode;             /* Character buffer code (Python v2). */
     codeBlock *picklecode;              /* Pickle code. */
     struct _classDef *next;             /* Next in the list. */
 } classDef;
@@ -950,7 +996,7 @@ typedef struct {
     codeBlock *docs;                    /* Documentation. */
     int sigslots;                       /* Set if signals or slots are used. */
     int genc;                           /* Set if we are generating C code. */
-    struct _stringList *options;        /* The list of options. */
+    struct _stringList *plugins;        /* The list of plugins. */
 } sipSpec;
 
 
@@ -987,15 +1033,17 @@ void warning(char *,...);
 void fatal(char *,...);
 void fatalScopedName(scopedNameDef *);
 int setInputFile(FILE *open_fp, parserContext *pc, int optional);
-void *sipMalloc(size_t);
+void *sipMalloc(size_t n);
+void *sipCalloc(size_t nr, size_t n);
 char *sipStrdup(const char *);
 char *concat(const char *, ...);
 void append(char **, const char *);
 void addToUsedList(ifaceFileList **, ifaceFileDef *);
 int excludedFeature(stringList *,qualDef *);
 int sameSignature(signatureDef *,signatureDef *,int);
-int sameTemplateSignature(signatureDef *sd1, signatureDef *sd2, int deep);
-int sameScopedName(scopedNameDef *,scopedNameDef *);
+int sameTemplateSignature(signatureDef *tmpl_sd, signatureDef *args_sd,
+        int deep);
+int compareScopedNames(scopedNameDef *snd1, scopedNameDef *snd2);
 int sameBaseType(argDef *,argDef *);
 char *scopedNameTail(scopedNameDef *);
 scopedNameDef *text2scopePart(char *);
@@ -1006,7 +1054,6 @@ void appendToClassList(classList **,classDef *);
 void appendCodeBlock(codeBlock **headp, codeBlock *new);
 void prcode(FILE *fp, const char *fmt, ...);
 void prOverloadName(FILE *fp, overDef *od);
-void prScopedPythonName(FILE *fp, classDef *scope, const char *pyname);
 void prOverloadDecl(FILE *fp, classDef *context, overDef *od, int defval);
 void searchTypedefs(sipSpec *pt, scopedNameDef *snd, argDef *ad);
 int isIntReturnSlot(memberDef *md);
@@ -1014,14 +1061,13 @@ int isLongReturnSlot(memberDef *md);
 int isVoidReturnSlot(memberDef *md);
 int isNumberSlot(memberDef *md);
 int isRichCompareSlot(memberDef *md);
-mappedTypeDef *allocMappedType(argDef *type);
+mappedTypeDef *allocMappedType(sipSpec *pt, argDef *type);
 void appendString(stringList **headp, const char *s);
 void appendTypeStrings(scopedNameDef *ename, signatureDef *patt, signatureDef *src, signatureDef *known, scopedNameDef **names, scopedNameDef **values);
 codeBlock *templateCode(sipSpec *pt, ifaceFileList **used, codeBlock *ocb, scopedNameDef *names, scopedNameDef *values);
 ifaceFileDef *findIfaceFile(sipSpec *pt, moduleDef *mod, scopedNameDef *fqname, ifaceFileType iftype, argDef *ad);
-int optNoEmitters(sipSpec *pt);
-int optRegisterTypes(sipSpec *pt);
-int optQ_OBJECT4(sipSpec *pt);
+int pluginPyQt3(sipSpec *pt);
+int pluginPyQt4(sipSpec *pt);
 void yywarning(char *);
 nameDef *cacheName(sipSpec *pt, const char *name);
 
@@ -1037,6 +1083,7 @@ typedef enum {
     string_flag,
     name_flag,
     opt_name_flag,
+    dotted_name_flag,
     integer_flag
 } flagType;
 
